@@ -1,7 +1,6 @@
 from ._version import __version__
-from .base import FileSet
+from .base import DataType, FileSet, Field
 from .utils import (
-    to_mime,
     from_mime,
     find_matching,
     MissingExtendedDependency,
