@@ -29,7 +29,6 @@ def test_magic_fail(work_dir):
     fspath = work_dir / "test.magic"
     write_test_file(fspath, b"NOMAGIC some contents\n\n", binary=True)
     assert not FileWithMagicNumber.matches(fspath)
-    assert FileWithMagicNumber.matches(fspath, validate=False)
 
 
 class Header(File):
@@ -68,8 +67,7 @@ def test_with_separate_header(work_dir):
     }
     write_test_file(hdr_fspath, "\n".join(f"{k}:{v}" for k, v in hdr.items()))
     file = FileWithSeparateHeader(fspath)
-    file.validate()
-    assert file.metadata.loaded == hdr
+    assert file.metadata == hdr
 
 
 def test_with_separate_header_fail1(work_dir):
