@@ -1,137 +1,19 @@
-from __future__ import annotations
 import importlib
-import re
 from warnings import warn
 from pathlib import Path
+import typing as ty
+import re
 import os
 import traceback
 import pkgutil
 from contextlib import contextmanager
 from fileformats.core.exceptions import (
-    FileFormatsError,
-    FormatRecognitionError,
     MissingExtendedDepenciesError,
 )
 import fileformats.core
 
 
-def to_mime(klass, iana=True):
-    """Generates a MIME (IANA) or "MIME-like" identifier from a format class (i.e.
-    an identifier for a non-MIME class in the MIME style), e.g.
-
-        fileformats.text.Plain to "text/plain"
-
-    and
-
-        fileformats.image.TiffFx to "image/tiff-fx"
-
-    Parameters
-    ----------
-    klass : type(FileSet)
-        FileSet subclass
-    iana_mime : bool
-        whether to use standardised IANA format or a more relaxed type format corresponding
-        to the fileformats extension the type belongs to
-
-    Returns
-    -------
-    type
-        the corresponding file format class
-    """
-    if iana and getattr(klass, "iana_mime", None) is not None:
-        return klass.iana_mime
-    format_name = to_mime_format_name(klass.__name__)
-    if iana:
-        mime = f"application/x-{format_name}"
-    else:
-        namespace_module = importlib.import_module("fileformats." + klass.namespace)
-        if getattr(namespace_module, klass.__name__, None) is not klass:
-            raise FileFormatsError(
-                f"Cannot create reversible MIME type for {klass} as it is not present in a "
-                f"top-level fileformats namespace package '{klass.namespace}'"
-            )
-        mime = f"{klass.namespace}/{format_name}"
-    return mime
-
-
-def from_mime(mime_string):
-    """Resolves a FileFormat class from a MIME (IANA) or "MIME-like" identifier (i.e.
-    an identifier for a non-MIME class in the MIME style), e.g.
-
-        "text/plain" resolves to fileformats.text.Plain
-
-    and
-
-        "image/tiff-fx" resolves to fileformats.image.TiffFx
-
-    Parameters
-    ----------
-    mime_string : str
-        MIME identifier
-
-    Returns
-    -------
-    type
-        the corresponding file format class
-    """
-    namespace, format_name = mime_string.split("/")
-    try:
-        return fileformats.core.FileSet.formats_by_iana_mime[mime_string]
-    except KeyError:
-        pass
-    if namespace == "application":
-        # We treat the "application" namespace as a catch-all for any formats that are
-        # not explicitly covered by the IANA standard (which is kind of how the IANA
-        # treats it). Therefore, we loop through all subclasses across the different
-        # namespaces to find one that matches the name.
-        if not format_name.startswith("x-"):
-            raise FormatRecognitionError(
-                "Did not find class matching official (i.e. non-extension) MIME type "
-                f"{mime_string} (i.e. one not starting with 'application/x-'"
-            ) from None
-        format_name = format_name[2:]  # remove "x-" prefix
-        matching_name = fileformats.core.FileSet.formats_by_name[format_name]
-        if not matching_name:
-            namespace_names = [n.__name__ for n in subpackages()]
-            class_name = from_mime_format_name(format_name)
-            raise FormatRecognitionError(
-                f"Did not find class matching extension the class name '{class_name}' "
-                f"corresponding to MIME type '{mime_string}' "
-                f"in any of the installed namespaces: {namespace_names}"
-            )
-        elif len(matching_name) > 1:
-            namespace_names = [f.__module__.__name__ for f in matching_name]
-            raise FormatRecognitionError(
-                f"Ambiguous extended MIME type '{mime_string}', could refer to "
-                f"{', '.join(repr(f) for f in matching_name)} installed types. "
-                f"Explicitly set the 'iana_mime' attribute on one or all of these types "
-                f"to disambiguate, or uninstall all but one of the following "
-                "namespaces: "
-            )
-        else:
-            klass = next(iter(matching_name))
-    else:
-        class_name = from_mime_format_name(format_name)
-        try:
-            module = importlib.import_module("fileformats." + namespace)
-        except ImportError:
-            raise FormatRecognitionError(
-                f"Did not find fileformats namespace package corresponding to {namespace} "
-                f"required to interpret '{mime_string}' MIME, or MIME-like, type. "
-                f"try installing the namespace package with "
-                f"'python3 -m pip install fileformats-{namespace}'."
-            ) from None
-        try:
-            klass = getattr(module, class_name)
-        except AttributeError:
-            raise FormatRecognitionError(
-                f"Did not find '{class_name}' class in fileformats.{namespace} "
-                f"corresponding to MIME, or MIME-like, type {mime_string}"
-            ) from None
-    return klass
-
-
-def find_matching(fspaths: list[Path], standard_only: bool = False):
+def find_matching(fspaths: ty.List[Path], standard_only: bool = False):
     """Detect the corresponding file format from a set of file-system paths
 
     Parameters
@@ -144,28 +26,18 @@ def find_matching(fspaths: list[Path], standard_only: bool = False):
     fspaths = fspaths_converter(fspaths)
     matches = []
     for frmt in fileformats.core.FileSet.all_formats:
-        if frmt.matches(fspaths) and frmt.namespace in STANDARD_NAMESPACES:
+        if frmt.matches(fspaths) and (
+            not standard_only or frmt.namespace in STANDARD_NAMESPACES
+        ):
             matches.append(frmt)
     return matches
 
 
-def to_mime_format_name(format_name):
-    format_name = format_name[0].lower() + format_name[1:]
-    format_name = re.sub("_([A-Z])", lambda m: "+" + m.group(1).lower(), format_name)
-    format_name = re.sub("([A-Z])", lambda m: "-" + m.group(1).lower(), format_name)
-    return format_name
+def from_mime(mime_str: str):
+    return fileformats.core.DataType.from_mime(mime_str)
 
 
-def from_mime_format_name(format_name):
-    if format_name.startswith("x-"):
-        format_name = format_name[2:]
-    format_name = format_name.capitalize()
-    format_name = re.sub(r"(-)(\w)", lambda m: m.group(2).upper(), format_name)
-    format_name = re.sub(r"(\+)(\w)", lambda m: "_" + m.group(2).upper(), format_name)
-    return format_name
-
-
-def splitext(fspath, multi=False):
+def splitext(fspath: Path, multi=False):
     """splits an extension from the file stem, taking into consideration multi-part
     extensions such as ".nii.gz".
 
@@ -210,7 +82,7 @@ def subpackages():
 
 
 @contextmanager
-def set_cwd(path):
+def set_cwd(path: Path):
     """Sets the current working directory to `path` and back to original
     working directory on exit
 
@@ -227,11 +99,15 @@ def set_cwd(path):
         os.chdir(pwd)
 
 
-def fspaths_converter(fspaths):
+def fspaths_converter(
+    fspaths: ty.Union[
+        ty.Iterable[ty.Union[str, os.PathLike, bytes]], str, os.PathLike, bytes
+    ]
+):
     """Ensures fs-paths are a set of pathlib.Path"""
-    if isinstance(fspaths, (str, Path, bytes)):
+    if isinstance(fspaths, (str, os.PathLike, bytes)):
         fspaths = [fspaths]
-    return set((Path(p) if isinstance(p, str) else p).absolute() for p in fspaths)
+    return frozenset(Path(p).absolute() for p in fspaths)
 
 
 class classproperty(object):
@@ -273,7 +149,7 @@ class MissingExtendedDependency:
         )
 
 
-def import_converters(module_name):
+def import_converters(module_name: str):
     """Attempts to import converters and raises warning if they can be imported"""
     try:
         importlib.import_module(module_name + ".converters")
@@ -302,3 +178,53 @@ STANDARD_NAMESPACES = [
     "text",
     "video",
 ]
+
+
+def to_mime_format_name(format_name: str):
+    format_name = format_name[0].lower() + format_name[1:]
+    format_name = re.sub("_([A-Z])", lambda m: "+" + m.group(1).lower(), format_name)
+    format_name = re.sub("([A-Z])", lambda m: "-" + m.group(1).lower(), format_name)
+    return format_name
+
+
+def from_mime_format_name(format_name: str):
+    if format_name.startswith("x-"):
+        format_name = format_name[2:]
+    format_name = format_name.capitalize()
+    format_name = re.sub(r"(-)(\w)", lambda m: m.group(2).upper(), format_name)
+    format_name = re.sub(r"(\+)(\w)", lambda m: "_" + m.group(2).upper(), format_name)
+    return format_name
+
+
+def hash_file(fspath: Path, chunk_len: int, crypto: ty.Callable):
+    crypto_obj = crypto()
+    with open(fspath, "rb") as fp:
+        for chunk in iter(lambda: fp.read(chunk_len), b""):
+            crypto_obj.update(chunk)
+    return crypto_obj.hexdigest()
+
+
+def hash_dir(
+    fspath: Path,
+    chunk_len: int,
+    crypto: ty.Callable,
+    ignore_hidden_files: bool = False,
+    ignore_hidden_dirs: bool = False,
+    relative_to: Path = None,
+):
+    if relative_to is None:
+        relative_to = fspath
+    file_hashes = {}
+    for dpath, _, filenames in sorted(os.walk(fspath)):
+        # Sort in-place to guarantee order.
+        filenames.sort()
+        dpath = Path(dpath)
+        if ignore_hidden_dirs and dpath.name.startswith(".") and str(dpath) != fspath:
+            continue
+        for filename in filenames:
+            if ignore_hidden_files and filename.startswith("."):
+                continue
+            file_hashes[str(dpath.relative_to(relative_to))] = hash_file(
+                dpath / filename, crypto=crypto, chunk_len=chunk_len
+            )
+    return file_hashes
